/*******************************************************************************
 *	@file			DLGraph.cpp
 *	@brief			DirectX Library
 *
 *	@date			21-Apr-2010(Wed)
 ******************************************************************************/

// vvZbT---------------------------------------------------------------
#include "DLGraph.h"

// O[oϐ---------------------------------------------------------

// ֐vg^Cv錾---------------------------------------------------

// -----------------------------------------------------------------------------
/*******************************************************************************
 *	@namespace	DL
 ******************************************************************************/
namespace	DL
{

/*******************************************************************************
 *	@fn				BOOL CDLGraph::SetViewPort( const SDLParam& SDL )
 *	@param[in]		SDL							DL\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			r[|[g̐ݒ
 ******************************************************************************/
BOOL CDLGraph::SetViewPort( const SDLParam& SDL )
{
	HRESULT hr = D3D_OK;
	// http://msdn.microsoft.com/ja-jp/library/cc323943.aspx
	m_D3DVP.X		= 0;
	m_D3DVP.Y		= 0;
	m_D3DVP.Width	= SDL.m_nWidth;
	m_D3DVP.Height	= SDL.m_nHeight;
	m_D3DVP.MinZ	= 0.0f;
	m_D3DVP.MaxZ	= 1.0f;
	hr = m_lpD3DDev->SetViewport( &m_D3DVP );
	if( FAILED( hr ) )
	{
		DXTRACE_ERR( "SetViewport()", hr );
		return FALSE;
	}

	return TRUE;
}

/*******************************************************************************
 *	@fn				BOOL CDLGraph::DeviceRestore( const HRESULT& hRes )
 *	@retval			none						Ȃ
 *	@brief			foCX̕
 ******************************************************************************/
BOOL CDLGraph::DeviceRestore( const HRESULT& hRes )
{
	// foCX̊mF
	if( SUCCEEDED( hRes ) ) return TRUE;

	HRESULT hr;
	// foCX̏
	hr = m_lpD3DDev->TestCooperativeLevel();
	if( FAILED( hr ) )
	{
		// foCX͂܂Ă
		if( hr == D3DERR_DEVICELOST )
			return TRUE;
		// \ʃG[
		if( hr != D3DERR_DEVICENOTRESET )
			return FALSE;

		// \[X

		// Ă݂
		hr = m_lpD3DDev->Reset( &m_D3DPP );
		if( FAILED( hr ) )
		{
			// foCX͂܂Ă
			if( hr == D3DERR_DEVICELOST )
				return TRUE;
			return FALSE;
		}

		// \[XĎ擾
	}

	return TRUE;
}

/*******************************************************************************
 *	@fn				CDLGraph::CDLGraph( void )
 *	@retval			none						Ȃ
 *	@brief			RXgN^
 ******************************************************************************/
CDLGraph::CDLGraph( void )
	: m_lpD3D( NULL )
	, m_lpD3DDev( NULL )

	, m_hWnd()
	, m_D3DPP()
	, m_D3DDM()
	, m_D3DVP()
{
}

/*******************************************************************************
 *	@fn				CDLGraph::~CDLGraph( void )
 *	@retval			none						Ȃ
 *	@brief			fXgN^
 ******************************************************************************/
CDLGraph::~CDLGraph( void )
{
}

/*******************************************************************************
 *	@fn				int CDLGraph::Init( const SDLParam& SDL )
 *	@param[in]		SDL							DL\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			
 ******************************************************************************/
int CDLGraph::Init( const SDLParam& SDL )
{
	m_hWnd = SDL.m_hWnd;
	HRESULT hr = D3D_OK;

	// IDirect3D9 C^[tFCX̎擾
	// http://msdn.microsoft.com/ja-jp/library/cc324303.aspx
	m_lpD3D = ::Direct3DCreate9( D3D_SDK_VERSION );
	if( m_lpD3D == NULL )
	{
		DXTRACE_MSG( "Direct3DCreate9()" );
		return FALSE;
	}

	// foCX\̂̏
	::ZeroMemory( &m_D3DPP, sizeof( D3DPRESENT_PARAMETERS ) );

	// ݂̉ʃ[h擾܂
	// http://msdn.microsoft.com/ja-jp/library/cc324298.aspx
	hr = m_lpD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &m_D3DDM );
	if( FAILED( hr ) )
	{
		DXTRACE_ERR( "GetAdapterDisplayMode()", hr );
		return FALSE;
	}

	// foCX\̂̐ݒ
	// http://msdn.microsoft.com/ja-jp/library/cc323931.aspx
	m_D3DPP.Windowed	= SDL.m_bMode;
	m_D3DPP.SwapEffect	= D3DSWAPEFFECT_DISCARD;
	m_D3DPP.BackBufferFormat	= m_D3DDM.Format;
	m_D3DPP.BackBufferWidth		= SDL.m_nWidth;
	m_D3DPP.BackBufferHeight	= SDL.m_nHeight;
	m_D3DPP.BackBufferCount		= 1;

	// foCX̊mF
	hr = m_lpD3D->CheckDeviceType( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_D3DPP.BackBufferFormat, m_D3DPP.BackBufferFormat, FALSE );
	if( FAILED( hr ) )
		DXTRACE_ERR( "CheckDeviceType()", hr );

	// IDirect3Device9 C^[tFCX̎擾
	// http://msdn.microsoft.com/ja-jp/library/cc324295.aspx
	// HAL + HARDWARE
	hr = m_lpD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_MULTITHREADED, &m_D3DPP, &m_lpD3DDev );
	if( FAILED( hr ) )
	{
		DXTRACE_ERR( "CreateDevice()", hr );
		// HAL + SOFTWARE
		hr = m_lpD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING | D3DCREATE_MULTITHREADED, &m_D3DPP, &m_lpD3DDev );
		if( FAILED( hr ) )
		{
			// REF + SOFTWARE ͎gpȂ
			DXTRACE_ERR( "CreateDevice()", hr );
			return FALSE;
		}
	}

	// r[|[g̐ݒ
	if( !SetViewPort( SDL ) )
		return FALSE;

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::End( void )
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			I
 ******************************************************************************/
int CDLGraph::End( void )
{
	// IDirect3Device9 C^[tFCX̉
	_RELEASE( m_lpD3DDev );
	// IDirect3D9 C^[tFCX̉
	_RELEASE( m_lpD3D );

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::Update( void )
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			ʍXV
 ******************************************************************************/
int CDLGraph::Update( void )
{
	HRESULT hr = D3D_OK;

	// V[̃NA
	hr = m_lpD3DDev->Clear( 0,
							NULL,
							D3DCLEAR_TARGET,
							D3DCOLOR_ARGB( 0x00, 0x00, 0x00, 0x00 ),
							1.0f,
							0 );
	if( FAILED( hr ) )
	{
		DXTRACE_ERR( "Clear()", hr );
		return FALSE;
	}

	// V[̕`Jn
	hr = m_lpD3DDev->BeginScene();
	if( SUCCEEDED( hr ) )
	{
		// V[̕`



		{
			static LPDIRECT3DTEXTURE9 m_pD3DTex = NULL;
			if( m_pD3DTex == NULL )
			{
				// 摜̓ǂݍ
				hr = D3DXCreateTextureFromFileEx(
						m_lpD3DDev,
						"setup.bmp",
						D3DX_DEFAULT,
						D3DX_DEFAULT,
						D3DX_DEFAULT,
						0,
						D3DFMT_UNKNOWN,
						D3DPOOL_MANAGED,
						D3DX_DEFAULT,
						D3DX_DEFAULT,
						0,
						NULL,
						NULL,
						&m_pD3DTex );
			}

			hr = m_lpD3DDev->SetTexture( 0, m_pD3DTex );

			// _Xg[̐錾ݒ
			hr = m_lpD3DDev->SetFVF( D3DFVF_XYZRHW | D3DFVF_DIFFUSE | D3DFVF_TEX1 );

			D3DVERTEX v[ 4 ] =
			{
				{   0.0f,   0.0f, 0.0f, 1.0f, 0xFFFF0000, 0.0f, 0.0f },					// 
				{ 800.0f,   0.0f, 0.0f, 1.0f, 0xFF00FF00, 1.0f, 0.0f },					// E
				{ 800.0f, 600.0f, 0.0f, 1.0f, 0xFF0000FF, 1.0f, 1.0f },					// E
				{   0.0f, 600.0f, 0.0f, 1.0f, 0xFF000000, 0.0f, 1.0f },					// 
			};
			// v~eBu`
			hr = m_lpD3DDev->DrawPrimitiveUP( D3DPT_TRIANGLEFAN,
											  4 - 2,
											  v,
											  sizeof( D3DVERTEX ) );
		}



		// V[̕`I
		m_lpD3DDev->EndScene();
	}

	// V[̕\
	hr = m_lpD3DDev->Present( NULL, NULL, NULL, NULL );
	// foCX̕
	if( !DeviceRestore( hr ) )
	{
		DXTRACE_ERR( "DeviceRestore()", hr );
		return FALSE;
	}

	return TRUE;
}

/*******************************************************************************
 *	@fn				int CDLGraph::ChangeDisplay( const SDLParam& SDL )
 *	@param[in]		SDL							DL\
 *	@retval			int							TRUE=OK/FALSE=NG
 *	@brief			ʐ؂ւ
 ******************************************************************************/
int CDLGraph::ChangeDisplay( const SDLParam& SDL )
{
	HRESULT hr = D3D_OK;

	// \TCY̐ݒ
	m_D3DPP.BackBufferWidth		= SDL.m_nWidth;
	m_D3DPP.BackBufferHeight	= SDL.m_nHeight;
	// rfI[hݒ
	m_D3DPP.Windowed			= SDL.m_bMode;

	// ʂ̐؂ւ
	hr = m_lpD3DDev->Reset( &m_D3DPP );
	if( !DeviceRestore( hr ) )
	{
		DXTRACE_ERR( "DeviceRestore()", hr );
		return FALSE;
	}

	// r[|[g̐ݒ
	if( !SetViewPort( SDL ) )
		return FALSE;

	return TRUE;
}

}	// namespace	DL
